# -*- coding: utf-8 -*-
"""
    openclip/const.py
    =======================================

    provides constants convenience functions: const, set_const
"""
import errno
import numbers
from . import constants as _c
from . import LOG
from openclip.exceptions import *

__all__ = [ 'const', 'set_const' ]


def const(const):
    """
    Convenience wrapper to yield the value of a constant
    based on module convention of importing constants as `_c`

    Usage:

    const("OPENCLIP_VERSION")
    returns "3"
    """
    try:
        return getattr(_c, const)
    except AttributeError:
        msg = u'No such constant: {0}'.format(const)
        LOG.critical(msg)
        raise OpenClipEnvError(msg)
    except TypeError:
        msg = u'const name must be a string or unicode object, not: {0}'.format(const.__class__.__name__)
        LOG.critical(msg)
        raise TypeError(msg)


def set_const(const, val):
    """
    Convenience wrapper to reliably set the value of a constant
    from outside of package scope based on the module convention
    of importing/exporting constants as `_c`

    Usage:

    set_const("OPENCLIP_VERSION", "4")

    sets the `_c.OPENCLIP_VERSION` to "4" and
    returns "4"
    """
    try:
        cur = getattr(_c, const)
    except AttributeError:
        msg = u'no such constant: {0}'.format(const)
        LOG.critical(msg)
        raise OpenClipEnvError(msg)
    except TypeError:
        msg = u'const name must be a string or unicode object, not: {0}'.format(const.__class__.__name__)
        LOG.critical(msg)
        raise TypeError(msg)
    should_be = cur.__class__
    try:
        if not isinstance(val, should_be):
            if should_be is unicode or cur is None:
                val = val
            elif should_be is int and const.endswith('MODE'):
                val = int(val, 8)
            elif isinstance(cur, numbers.Integral):
                val = int(val)
            else:
                should_be(val)
    except (TypeError, ValueError, ):
        msg = u'invalid type for constant {0}, should be {1}, not: {2}'.format(const, should_be.__name__, val.__class__.__name__)
        LOG.critical(msg)
        raise OpenClipEnvError(msg)
    setattr(_c, const, val)
    return val
